@@ -1,6 +1,6 @@
 package UvT::Email::Template;
 
-# $Id: Template.pm 39337 2013-05-03 09:42:34Z wsl $
+# $Id: Template.pm 39364 2013-05-06 09:34:28Z wsl $
 # $URL: https://svn.uvt.nl/its-id/trunk/sources/uvt-email-template/lib/UvT/Email/Template.pm $
 
 # formatteer een XML-bericht tot een multipart/alternative MIME
@@ -14,6 +14,7 @@ use Clarity -self;
 use XML::LibXML;
 use XML::LibXSLT;
 use Email::MIME::Creator;
+use MIME::Types;
 use Text::Tabs qw(expand);
 use Scalar::Util qw(weaken reftype refaddr);
 
@@ -185,6 +186,7 @@ sub format_table {
 			unless $name eq 'tr';
 		my @c;
 		foreach my $c ($r->childNodes) {
+			next unless $r->nodeType == XML_ELEMENT_NODE;
 			my $name = $c->nodeName // next;
 			die "<tr> element must only contain <th> or <td> elements\n"
 				unless $name eq 'th' || $name eq 'td';
@@ -391,13 +393,6 @@ field xml => sub {
 
 field xslt => sub { new XML::LibXSLT };
 
-field mimetypes => {
-	gif => 'image/gif',
-	png => 'image/png',
-	jpg => 'image/jpeg',
-	jpeg => 'image/jpeg',
-};
-
 sub htmltemplate {
 	return $self->{htmltemplate} unless @_;
 	my $t = shift;
@@ -416,6 +411,8 @@ sub transformer {
 	return $self->{transformer} = $t;
 }
 
+field mimetypes => sub { new MIME::Types };
+
 sub format {
 	my $doc = shift;
 
@@ -438,7 +435,6 @@ sub format {
 	my $xpc = new XML::LibXML::XPathContext;
 	$xpc->registerNs(html => 'http://www.w3.org/1999/xhtml');
 
-	my $mimetypes = $self->mimetypes;
 	my $imagepath = $self->imagepath;
 	$imagepath = [$imagepath] unless ref $imagepath;
 
@@ -447,18 +443,24 @@ sub format {
 			// die "img element has no 'src' attribute\n";
 		next if $src =~ /^[a-z0-9_-]+:/i;
 		die "Refusing to open image file with funny name ($src)\n"
-			if $src !~ /^[a-z0-9]+(?:[_\/.-][a-z0-9]+)*\.([a-z0-9]+(?:[_\/-][a-z0-9]+)*)$/;
-		my $type = $mimetypes->{lc($1)};
-		die "Unknown image extension '$1'\n"
-			unless defined $type;
+			unless $src =~ /^[a-z0-9]+(?:[_\/.-][a-z0-9]+)*$/;
+
 		my $path;
 		foreach my $p (@$imagepath) {
-			next unless -r "$p/$src";
+			next unless -f "$p/$src";
 			$path = "$p/$src";
 			last;
 		}
 		die "Image '$src' not found in imagepath [@$imagepath]\n"
 			unless defined $path;
+
+		my $mimetype = $self->mimetypes->mimeTypeOf($path);
+		die "Unable to determine mime type for '$path'\n"
+			unless defined $mimetype;
+		my $type = $mimetype->type;
+		die "File '$path' ($type) is not an image\n"
+			unless $mimetype->mediaType eq 'image';
+
 		my $id = @attachments + 1;
 		$img->setAttribute(src => "cid:$id");
 		push @attachments, Email::MIME->create(
@@ -510,7 +512,7 @@ UvT::Email::Format - UvT style email formatting
 =head1 INTRODUCTION
 
 This class can create ready to use e-mails formatted using XSL templates
-(for HTML) or a built-in XML converter (for text). The resulting MIME
+(for HTML) and a built-in XML converter (for text). The resulting MIME
 structure is a multipart/alternative structure containing both the HTML
 and text versions.
 
@@ -537,11 +539,6 @@ This field contains the XML parser options that are used to parse any XML.
 It's a hash that you can either replace or modify. See XML::LibXML::Parser
 for details.
 
-=head2 mimetypes
-
-A hash that maps file extensions to MIME types. You can replace or modify
-it.
-
 =head2 transformer
 
 An optional stylesheet that will be applied to your XML document when you
@@ -645,7 +642,7 @@ The allowed html elements and attributes are:
 
 =back
 
-Unless noted, no attributes are allowed.
+Unless noted otherwise, no attributes are allowed.
 
 =head1 AUTHOR
 
